package r;

import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Point;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.RMainLoopCallbacks;
import org.rosuda.JRI.Rengine;

import util.VisDataNode;
import util.VisPlotNode;
import util.VisTreeNode;

public class RManager implements RMainLoopCallbacks {
	public Rengine re;

	public RManager(Rengine re) {
		this.re = re;
	}

	// public void load(File outpath){
	// re.eval("load(\"" + outpath.getPath() + "\")", false);
	// }

	public void unload(String name) {
		execute(new String[] { "rm(\"" + name + "\")" });
	}

	public void saveWorkspace(File outpath) {
		execute(new String[] { "save.image(\"" + outpath.getPath()
				+ "\",compress=TRUE)" });
	}

	public void clearWorkspace() {
		execute(new String[] { "rm(list=ls())" });
	}

	public String[] getDataList() {
		REXP result = re.eval("ls()");
		return result.asStringArray();
	}
	
	public String getFlowNumber(String dataname){
		String number = "";
		REXP result = re.eval("dim(" + dataname + ")[1]");
		if (result.getType() == REXP.XT_ARRAY_INT)
			number += result.asIntArray()[0];
		else if (result.getType() == REXP.XT_ARRAY_DOUBLE)
			number += result.asDoubleArray()[0];
		
		return number;
	}
	
	public String getFlowNumber(VisDataNode data){
		String number = "";
		int level = data.getLevel();
		REXP result;
		if (level==1)
			result = re.eval("dim(" + data.name + ")[1]");
		else if (level > 1)
			result = re.eval("dim(iset(\"" + data.name + "\")[])[1]");
		else
			return number;
		
		if (result.getType() == REXP.XT_ARRAY_INT)
			number += result.asIntArray()[0];
		else if (result.getType() == REXP.XT_ARRAY_DOUBLE)
			number += result.asDoubleArray()[0];

		return number;
	}

	private String array2String(int[] array) {
		String str = "";
		for (int i = 0; i < array.length - 1; i++)
			str += array[i] + ".0,";
		str += array[array.length - 1] + ".0";
		return str;
	}

	private String array2String(double[] array) {
		String str = "";
		for (int i = 0; i < array.length - 1; i++)
			str += array[i] + ",";
		if (array.length > 0)
			str += array[array.length - 1];
		return str;
	}

	// private String vector2String(RVector vector){
	// String str = "";
	// Iterator i = vector.iterator();
	// while(i.hasNext())
	// System.err.println(((REXP)i.next()).toString());
	//		
	// str = array2String(((REXP)(vector.get(0))).asDoubleArray());
	// return str;
	// }

	private String getIndexString(REXP result) {
		String str = "";
		int type = result.getType();

		System.err.println("index result type is: " + REXP.xtName(type));

		if (type == REXP.XT_VECTOR) {
			result = (REXP) (result.asVector().get(0));
			type = result.getType();
		}

		if (type == REXP.XT_ARRAY_INT) {
			str = array2String(result.asIntArray());
		} else if (type == REXP.XT_ARRAY_DOUBLE) {
			str = array2String(result.asDoubleArray());
		} else {
			System.err.println("Unhandled REXP in getIndexString().");
		}

		return str;
	}

	public Vector getIndex(String filter, VisDataNode parent,
			boolean isFilterSelection) {
		Vector ret = new Vector();
		String index = "";

		String rootDataName = parent.getRootDataNode().name;

		String[] cmds = null;

		int resultRow;

		if (parent.getLevel() == 1) {
			if (filter.equals("")) {
				ret.addElement("ALL");
				ret.addElement(parent);
				return ret;
			}
			if (isFilterSelection){
				if (filter.equals("conn")){
					filter = "filter_connection(" + rootDataName + ")";
				}else if (filter.equals("!conn")){
					filter = "!filter_connection(" + rootDataName + ")";
				}
			}
				String selection = isFilterSelection ? "which(" + filter + ")"
						: "c(" + filter + ")";
				cmds = new String[] { "source(\"scripts/filter_connection.r\")", "attach(" + rootDataName + ");",
						selection + ";", "detach(" + rootDataName + ")", "rm(filter_connection)" };
				resultRow = 2;
		} else if (parent.getLevel() > 1) {
			if (filter.equals("")) {
				ret.addElement(parent.originalIndex);
				parent = (VisDataNode) (parent.getParent());
				ret.addElement(parent);
				return ret;
			}
			
			String parentData = (parent.originalIndex.equals("ALL")) ? rootDataName
					: rootDataName + "[c(" + parent.originalIndex + "),]";
			if (isFilterSelection){
				if (filter.equals("conn")){
					filter = "filter_connection(" + parentData + ")";
				}else if (filter.equals("!conn")){
					filter = "!filter_connection(" + parentData + ")";
				}
			}
			String selection = isFilterSelection ? "which(" + filter + ")"
					: "c(" + filter + ")";

			cmds = new String[] {
					"source(\"scripts/filter_connection.r\")",
					"parentData <- " + parentData + ";",
					"attach(parentData);",
					"iset.set(\'" + parent.name + "\')",
					"iset(\"" + parent.name + "\")" + "[" + selection + ",'index']",
					"detach(parentData)",
					"rm(parentData)",
					"rm(filter_connection)"};
			resultRow = 4;
		} else {
			System.err.println("Invalid parent node in getIndex().");
			return null;
		}
		REXP[] result = null;
		if (cmds != null)
			result = execute(cmds);

		if (result != null)
			index = getIndexString(result[resultRow]);

		ret.addElement(index);
		ret.addElement(parent);
		return ret;
	}

	public boolean createISet(VisDataNode parent, VisDataNode child) {
		String rootDataName = parent.getRootDataNode().name;
		String data = child.originalIndex.equals("ALL") ? rootDataName
				: rootDataName + "[c(" + child.originalIndex + "),]";
		String index = child.originalIndex.equals("ALL") ? "1:dim("
				+ rootDataName + ")[1]" : child.originalIndex;

		String[] cmds = new String[] {
				"iset.new(\"" + child.name + "\", " + data + ");",
				"ivar.new(\"index\", c(" + index + "))" };

		if (execute(cmds) == null)
			return false;

		System.err.println("iSet: " + child.name + " is created over " + data);

		return true;
	}

	public boolean setISet(String name) {
		if (re.eval("iset.set(\"" + name + "\")") == null)
			return false;
		return true;
	}
	
	public void brush(VisDataNode node, String attribute) {
		String[] cmds = new String[]{
				"source(\"scripts/rainbowcolor.r\")",
				"iset.set(\"" + node.name + "\")",
				"iset.col(rainbowcolor(iset(\"" + node.name + "\")[]$" + attribute + "))",
				"rm(rainbowcolor)"
		};
		if (execute(cmds) == null)
			System.err.println("Rainbow brush " + node.name + " over " + attribute + " failed.");
	}

	public void brush(VisDataNode node, Hashtable<String, String> color) {
		String[] cmds = getBrushCmds(node, color);
		if (execute(cmds) == null)
			System.err.println("Brush " + node.name + " failed.");
	}

	public String[] getBrushCmds(VisDataNode node,
			Hashtable<String, String> color) {
		String rootDataName = node.getRootDataNode().name;
		String data = (node.originalIndex.equals("ALL")) ? rootDataName
				: rootDataName + "[c(" + node.originalIndex + "),]";
		String[] cmds = new String[color.size() + 7];
		cmds[0] = "iset.set(\"" + node.name + "\")";
		cmds[1] = "color <- rep(" + color.get("DEFAULT")
				+ ", dim(iset(iset.cur()))[1])";
		cmds[2] = "data <- " + data;
		cmds[3] = "attach(data)";
		
		Vector<String> sortedKeys = new Vector<String>();
		sortedKeys.addAll(color.keySet());
		Collections.sort(sortedKeys);
		int ctr = 4;
		for (int i = 0; i< sortedKeys.size(); i++){
			String key = (String) sortedKeys.get(i);
			if (key.equals("DEFAULT"))
				continue;
			cmds[ctr++] = "color[" + color.get(key) + "] <- " + key;
		}
		cmds[cmds.length - 4] = "detach(data)";
		cmds[cmds.length - 3] = "rm(data)";
		// cmds[cmds.length-2] = "iset.col(color)";
		cmds[cmds.length - 2] = "iplot.opt(col=color)";
		cmds[cmds.length - 1] = "rm(color)";
		return cmds;
	}

	public String getSelectedIndex(String name) {
		String sel = "";

		String[] cmds = new String[] { "iset.set(\"" + name + "\")",
				"iset.selected()"
		// "iset(\"" + name + "\")[iset.selected(),'index']"
		};

		REXP[] result = execute(cmds);

		if (result != null) {
			sel = getIndexString(result[1]);
		}

		return sel;
	}

	public void togglePlot(VisPlotNode plot) {
		VisDataNode p = (VisDataNode) plot.getParent();
		plot.isVisible = !plot.isVisible;
		String isVisible = plot.isVisible ? "TRUE" : "FALSE";
		String state = plot.isVisible ? "0" : "1";
		String[] cmds = new String[] {
				"iset.set(\"" + p.name + "\")",
				"iplot.set(" + plot.id + ")",
				".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setState\",as.integer("
						+ state + "))",
				// ".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setAlwaysOnTop\","
				// + isVisible + ")"
				".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setVisible\","
						+ isVisible + ")" };
		execute(cmds);
	}

	public void showPlots(VisDataNode parent, boolean show) {
		if (parent.getPlotCount() == 0)
			return;

		parent.isVisible = show;
		String isVisible = parent.isVisible ? "TRUE" : "FALSE";
		String state = parent.isVisible ? "0" : "1";
		execute(new String[] { "iset.set(\"" + parent.name + "\")" });

		VisPlotNode p;
		String[] cmds;
		for (int i = 0; i < parent.getChildCount(); i++) {
			if (!(parent.getChildAt(i) instanceof VisPlotNode))
				continue;
			p = (VisPlotNode) parent.getChildAt(i);
			if (p.isVisible == parent.isVisible)
				continue;

			p.isVisible = !p.isVisible;
			cmds = new String[] {
					"iset.set(\"" + parent.name + "\")",
					"iplot.set(" + p.id + ")",
					".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setState\",as.integer("
							+ state + "))",
					// ".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setAlwaysOnTop\","
					// + isVisible + ")",
					".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setVisible\","
							+ isVisible + ")" };
			execute(cmds);
		}
	}

	public void togglePlots(VisDataNode parent) {
		if (parent.getPlotCount() == 0)
			return;

		parent.isVisible = !parent.isVisible;
		showPlots(parent, parent.isVisible);
	}

	public void tilePlots(VisDataNode parent, Point ori, Dimension size) {
		System.out.println(ori + " : " + size);
		int plotCount;
		if (parent == null || (plotCount = parent.getPlotCount()) == 0
				|| ori == null || size == null)
			return;

		int offsetx = 0;
		int offsety = 12;

		int x = ori.x;
		int y = ori.y;
		double sqrt = Math.sqrt(plotCount);
		long roundSqrt = Math.round(sqrt);
		long n = roundSqrt < sqrt ? roundSqrt + 1 : roundSqrt;
		long width = size.width / n - offsetx;
		long height = size.height / n - offsety;

		int cmdCountEach = 1;
		String[] cmds = new String[plotCount * cmdCountEach + 1];

		cmds[0] = "iset.set(\"" + parent.name + "\")";
		for (int i = 0; i < (cmds.length - 1) / cmdCountEach; i++) {
			cmds[i * cmdCountEach + 1] = ".jcall(.jcall(.iplots[[as.integer("
					+ (i + 1)
					+ ")]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),,\"setBounds\", as.integer("
					+ x + "), as.integer(" + y + "), as.integer(" + width
					+ "), as.integer(" + height + "))";
			System.out.println(i + ":" + x + ":" + y);
			
			long distance = (ori.x + size.width) - (x + width); 
			if (distance > width - size.width % n ) {
				x += width;
			}else if (distance == width - size.width % n){
				x += width - size.width % n;
			}else{
				x = ori.x;
				y += height;
			}
		}
		execute(cmds);

		showPlots(parent, true);
	}

	public String[] getVisCmd(VisPlotNode plot) {
		VisDataNode parent=(VisDataNode) plot.getParent();
		String rootDataName = parent.getRootDataNode().name;
		String plotArgs = "";
		if (plot.type == VisPlotNode.ScatterPlot) {
//			plotArgs = "iset(\"" + parent.name + "\")" + "$" + plot.attributes[0]
//					+ "," + "iset(\"" + parent.name + "\")" + "$"
//					+ plot.attributes[1] + ", customFieldBg=TRUE, COL_CUSTOMBG=\""
//					+ plot.background + "\"" + ", ptDiam=1";
			plotArgs = plot.getAttribute() + ", customFieldBg=TRUE, COL_CUSTOMBG=\""
			                                               					+ plot.background + "\"" + ", ptDiam=1";
		}else if (plot.type == VisPlotNode.MosaicPlot){
			plotArgs = "as.factor(" + plot.getAttribute().replace(",", "), as.factor(") + ")";
		}else {
			plotArgs = plot.getAttribute();
		}

		// String[] brushCmds = getBrushCmds(parent, color);
		String attribute = plot.getAttribute();

		String isVisible = plot.isVisible ? "TRUE" : "FALSE";
		String state = plot.isVisible ? "0" : "1";
		String size = plot.isVisible ? "400, 300" : "1,1";
		String fg = plot.foreground.equals("") ? "" : (plot.type == VisPlotNode.ScatterPlot ? "fillColor=\"" + plot.foreground + "\"" : "borderColor=\"" + plot.foreground + "\"");

		// int extra = 3;
		// String[] cmds = new String[brushCmds.length + extra];
		String[] cmds = new String[9];
		cmds[0] = "iset.set(\'" + parent.name + "\')";
		cmds[1] = "attach(iset(\"" + parent.name + "\")[])";
		cmds[2] = VisPlotNode.PLOTNAME[plot.type] + "(" + plotArgs + ")";
		cmds[3] = "detach(iset(\"" + parent.name + "\")[])";

		cmds[4] = ".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setState\",as.integer("
				+ state + "))";
		cmds[5] = ".jcall(.jcall(.iplots[[as.integer(iplot.cur())]]$obj,\"Ljava/awt/Frame;\",\"getFrame\"),\"V\",\"setVisible\","
				+ isVisible + ")";
		cmds[6] = "iplot.size(" + size + ")";
		cmds[7] = "iplot.opt(title=\"" + plot.name + "_"
				+ VisPlotNode.PLOTNAME[plot.type] + "(" +  attribute + ")\")";
		// for (int i = extra; i < cmds.length; i++)
		// cmds[i] = brushCmds[i - extra];
		cmds[8] = "iplot.opt(" + fg + ")";
		return cmds;
	}

	public void deletePlot(VisDataNode n) {
		int childCount;
		if (n == null || (childCount = n.getChildCount()) == 0)
			return;

		VisTreeNode child;
		for (int i = 0; i < childCount; i++) {
			if (((child = (VisTreeNode) n.getChildAt(i)) instanceof VisDataNode))
				deletePlot((VisDataNode) child);
		}
		String[] cmds = new String[] { "iset.rm(iset(\"" + n.name + "\"))" };

		execute(cmds);
	}

	public void savePlot(VisPlotNode plot, String dir) {
		if (plot == null || !(new File(dir).isDirectory()))
			return;

		VisDataNode parent = (VisDataNode) plot.getParent();
		if (parent == null)
			return;
		String file = dir + plot.description;
		String width = "1600";
		String height = "1200";
		String labelPosition = "";
//		if (plot.task.plot == VisTask.ScatterPlot) {
//			if (!plot.task.attributes[0].equals("tss")
//					&& !plot.task.attributes[0].equals("tes")
//					&& !plot.task.attributes[0].equals("prs")
//					&& !plot.task.attributes[0].equals("sas")
//					&& !plot.task.attributes[0].equals("das")
//					&& !plot.task.attributes[0].equals("flgs"))
//				labelPosition = "min(iset(iset.cur())$"
//						+ plot.task.attributes[0] + "[]),";
//			else
//				labelPosition = "0,";
//			if (!plot.task.attributes[1].equals("tss")
//					&& !plot.task.attributes[1].equals("tes")
//					&& !plot.task.attributes[1].equals("prs")
//					&& !plot.task.attributes[1].equals("sas")
//					&& !plot.task.attributes[1].equals("das")
//					&& !plot.task.attributes[1].equals("flgs"))
//				labelPosition += "max(iset(iset.cur())$"
//						+ plot.task.attributes[1] + "[])";
//			else
//				labelPosition += "nlevels(iset(iset.cur())$"
//						+ plot.task.attributes[1] + "[])";
//		}

		String cmds[] = new String[] {
				"iset.set(\"" + parent.name + "\")",
				"iplot.set(" + plot.id + ")",
				"p<-.iplots[[as.integer(iplot.cur())]]",
//				"itext(" + labelPosition + ",\"" + plot.getDescription()
//						+ "," + ((VisDataNode)plot.getParent()).getDescription() + "\")",
				"com=.jcall(p$obj,\"Ljava/awt/Component;\",\"getComponent\")",
				"h<-.jcall(com,\"I\",\"getHeight\")",
				"w<-.jcall(com,\"I\",\"getWidth\")",
				".jcall(com,,\"setSize\", as.integer(" + width
						+ "), as.integer(" + height + "))",
				"img=.jnew(\"java/awt/image/BufferedImage\",as.integer("
						+ width + "),as.integer(" + height + "),as.integer(1))",
				"g=.jcall(img,\"Ljava/awt/Graphics;\",\"getGraphics\")",
				".jcall(com,,\"paint\",g)",
				".jcall(\"javax/imageio/ImageIO\",\"Z\",\"write\",.jcast(img,\"java/awt/image/RenderedImage\"),\"png\",.jnew(\"java/io/File\", \""
						+ file + ".png\"))",
				".jcall(com,,\"setSize\", as.integer(w), as.integer(h))",
				"rm(p)", "rm(h)", "rm(w)", "rm(com)", "rm(img)", "rm(g)" };
		execute(cmds);
	}

	public void savePlot(VisDataNode parent, String dir) {
		if (parent == null  || !(new File(dir).isDirectory()))
			return;
		
		File f = new File(dir + parent.name + ".info");
		try {
			BufferedWriter out = new BufferedWriter(
					new FileWriter(f));
			out.write(parent.getDescription());
			out.close();
		} catch (IOException ioe) {
			System.err
					.println("Failed writing filter expression to file : "
							+ f.getPath() + ". Ex: " + ioe);
			ioe.printStackTrace();
		}

		VisTreeNode child;
		for (int i = 0; i < parent.getChildCount(); i++) {
			if ((child = (VisTreeNode) parent.getChildAt(i)) instanceof VisPlotNode)
				savePlot((VisPlotNode) child, dir);
		}
	}

	public void saveAllPlot(VisTreeNode parent, String dir) {
		if (parent == null  || !(new File(dir).isDirectory()))
			return;

		VisTreeNode child;
		for (int i = 0; i < parent.getChildCount(); i++) {
			if ((child = (VisTreeNode) parent.getChildAt(i)) instanceof VisDataNode)
				saveAllPlot((VisDataNode) child, dir);
		}

		if (parent instanceof VisDataNode)
			savePlot((VisDataNode) parent, dir);
	}

	public void deletePlot(VisPlotNode n) {
		if (n == null)
			return;
		VisDataNode p = (VisDataNode) n.getParent();
		String[] cmds = new String[] { "iset.set(\"" + p.name + "\")",
				"iplot.off(" + n.id + ")" };

		for (int i = p.getIndex(n) + 1; i < p.getChildCount(); i++)
			if (p.getChildAt(i) instanceof VisPlotNode)
				((VisPlotNode) p.getChildAt(i)).id--;
		execute(cmds);
	}

	public String[] getLoadWorkspaceCmd(File outpath) {
		return new String[] { "load(\"" + outpath.getPath() + "\")" };
	}

	public REXP[] execute(String[] cmds) {
		REXP[] result = new REXP[cmds.length];
		boolean succeed = true;
		int i;
		for (i = 0; i < cmds.length; i++) {
			result[i] = re.eval(cmds[i]);
			if (result[i] == null) {
				succeed = false;
			}
		}
		int lengthLimit = 200;
		String cmd = "";
		for (int j = 0; j < cmds.length - 1; j++) {
			cmd += j + "> " + cmds[j].substring(0, (cmds[j].length() > lengthLimit ? lengthLimit
					: cmds[j].length())) + "\n";
			cmd += j + ">> ";
			cmd += result[j] == null ? "NULL\n"
					: result[j]
							.toString()
							.substring(
									0,
									result[j].toString().length() > lengthLimit ? lengthLimit
											: result[j].toString().length())
							+ "\n";
		}
		cmd += (cmds.length - 1) + "> " + cmds[cmds.length - 1].substring(0, (cmds[cmds.length - 1].length() > lengthLimit ? lengthLimit
				: cmds[cmds.length - 1].length())) + "\n";
		cmd += cmds.length - 1 + ">> ";
		cmd += result[cmds.length - 1] == null ? "NULL\n"
				: result[cmds.length - 1]
						.toString()
						.substring(
								0,
								result[cmds.length - 1].toString().length() > lengthLimit ? lengthLimit
										: result[cmds.length - 1].toString()
												.length())
						+ "\n";
		if (succeed)
			System.err.println("R commands:\n" + cmd + " execution succeeded.");
		else
			System.err.println("R commands:\n" + cmd
					+ " execution failed at line No. " + (i - 1));

		return succeed ? result : null;
	}

	public void rWriteConsole(Rengine re, String text, int oType) {
		System.out.print(text);
	}

	public void rBusy(Rengine re, int which) {
		System.out.println("rBusy(" + which + ")");
	}

	public String rReadConsole(Rengine re, String prompt, int addToHistory) {
		System.out.print(prompt);
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(
					System.in));
			String s = br.readLine();
			return (s == null || s.length() == 0) ? s : s + "\n";
		} catch (Exception e) {
			System.out.println("jriReadConsole exception: " + e.getMessage());
		}
		return null;
	}

	public void rShowMessage(Rengine re, String message) {
		System.out.println("rShowMessage \"" + message + "\"");
	}

	public String rChooseFile(Rengine re, int newFile) {
		FileDialog fd = new FileDialog(new Frame(),
				(newFile == 0) ? "Select a file" : "Select a new file",
				(newFile == 0) ? FileDialog.LOAD : FileDialog.SAVE);
		fd.show();
		String res = null;
		if (fd.getDirectory() != null)
			res = fd.getDirectory();
		if (fd.getFile() != null)
			res = (res == null) ? fd.getFile() : (res + fd.getFile());
		return res;
	}

	public void rFlushConsole(Rengine re) {
	}

	public void rLoadHistory(Rengine re, String filename) {
	}

	public void rSaveHistory(Rengine re, String filename) {
	}
}
