package shell;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import util.Task;

public class ShellManager {	
	public int execute(String[] cmds, StringBuffer output) throws IOException, InterruptedException {
		Runtime runtime = Runtime.getRuntime();
		Process proc = runtime.exec(cmds);
		try {
			// put a BufferedReader on the execution output

			InputStream inputstream = proc.getInputStream();
			InputStreamReader inputstreamreader = new InputStreamReader(
					inputstream);
			BufferedReader bufferedreader = new BufferedReader(
					inputstreamreader);

			// read the execution output

			String line;
			while ((line = bufferedreader.readLine()) != null) {
				System.out.println(line);
				if (output != null) output.append(line); 
			}

			// check for execution failure

			try {
				if (proc.waitFor() != 0) {
					System.err.println("Shell command " + cmds[0]
							+ " execution failed. Exit value = "
							+ proc.exitValue());
				}
			} catch (InterruptedException e) {
				System.err.println("Shell command " + cmds[0]
						+ " execution interrupted." + e);
				throw e;
			}
		} catch (IOException e) {
			System.out.println("EX:" + e);
			e.printStackTrace();
			throw e;
		}
		return proc.exitValue();
	}

	public String[] getDumpCmd(String indir, String infile, String inpath,
			String timewindow, String filter, String outpath) {

		String[] cmd = new String[] { "scripts/dump.sh", outpath,
				indir != "" ? "-M" : "", indir, infile != "" ? "-R" : "",
				infile, inpath != "" ? "-r" : "", inpath,
				timewindow != "" ? "-t" : "", timewindow, 
						filter != "" ? "-f" : "", filter };
		return cmd;
	}

	public String[] getTimewindowCmd(String indir, String infile, String inpath) {

		String[] cmd = new String[] { "scripts/gettimewindow.sh", 
				indir != "" ? "-M" : "", indir, infile != "" ? "-R" : "",
				infile, inpath != "" ? "-r" : "", inpath};
		return cmd;
	}
	
	public String[] getRPreloadCmd(String outpath){
		File f = new File(outpath);
		String[] cmd = new String[]{"scripts/preload.sh", f.getPath(), f.getName()};
		return cmd;
	}
}
