package util;

import java.io.File;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

public class FileTreeModel extends DefaultTreeModel {
	public FileTreeModel(File rootPath) {
		super(new DefaultMutableTreeNode(rootPath));
		reload();
	}
	
	public void reload(){
		reload((TreeNode)getRoot());
	}
	
	public void reload(TreeNode parent){
		this.sortChildren(parent);
		super.reload(parent);
	}

	public Object getChild(Object parent, int index) {
		this.sortChildren(parent);
		return super.getChild(parent, index);
	}

	public int getChildCount(Object parent) {
		this.sortChildren(parent);
		return super.getChildCount(parent);
	}

	public int getIndexOfChild(Object parent, Object child) {
		this.sortChildren(parent);
		return super.getIndexOfChild(parent, child);
	}
	
	public void setRoot(File f){
		super.setRoot(new DefaultMutableTreeNode(f));
	}

	public boolean isLeaf(Object node) {
		File n = (File) ((DefaultMutableTreeNode) node).getUserObject();
		return n.isFile();
	}

	private void sortChildren(Object parent) {
		if (parent == null)
			return;

		DefaultMutableTreeNode node = (DefaultMutableTreeNode) parent;
		File file = (File) node.getUserObject();
		Vector<String> children = new Vector<String>();
		String[] tmp = file.list();
		if (tmp==null) return;
		for (int i = 0; i < tmp.length; i++)
			children.addElement(tmp[i]);
		Collections.sort(children);

		node.removeAllChildren();
		Iterator childrenIterator = children.iterator();
		while (childrenIterator.hasNext()) {
			DefaultMutableTreeNode newChild =new DefaultMutableTreeNode(new File(file.getPath()
					+ File.separator + (String) childrenIterator.next())); 
			node.add(newChild);
			if (!newChild.isLeaf())
				sortChildren(newChild);
		}
	}
}
