package util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Vector;

public class TaskManager extends Thread {
	Vector<Task> taskList = new Vector<Task>();

	Vector<TaskListener> taskListeners = new Vector<TaskListener>();

	boolean isAlive = true;

	public void run() {
		while (isAlive) {
			Task t = getNextQueuedTask();
			executeTask(t);
		}
	}

	public void addTask(Task t) {
		if (t == null) return;
		t.status = Task.QUEUED;
		taskList.addElement(t);
		System.out.println(Task.TYPENAME[t.type] + " "
				+ Task.STATUSNAME[t.status]);
		notifyManager(t);
	}

	public void removeTask(Task t) {
		if (t == null || t.status == Task.INPROGRESS) return;
		taskList.removeElement(t);
		notifyManager(t);
	}

	public void cancellTask(Task t) {
		if (t == null) return;
		if (t.status != Task.INPROGRESS)
			t.status = Task.CANCELLED;
		notifyManager(t);
	}

	public void restartTask(Task t) {
		if (t == null || t.status == Task.INPROGRESS)
			return;
		t.status = Task.QUEUED;
		notifyManager(t);
	}

	public void notifyManager(Task t) {
		synchronized (this) {
			this.notifyAll();
			this.notifyTaskListeners(t);
		}
	}

	private String getCurrentTime() {
		String time;
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd.HH:mm:ss");
		time = sdf.format(cal.getTime());
		return time;
	}

	private String getDuration(String start, String end) {
		String duration = "N/A";
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd.HH:mm:ss");
		try {
			duration = String.valueOf((sdf.parse(end).getTime() - sdf.parse(
					start).getTime()) / 1000);
		} catch (ParseException e) {
			System.err
					.println("Parsing failed when try to duration of task. EX:"
							+ e);
			e.printStackTrace();
		}
		return duration;
	}

	private Task getNextQueuedTask() {
		Task next = null;
		Iterator i;
		synchronized (this) {
			while (next == null) {
				while (taskList.isEmpty()) {
					try {
						this.wait();
					} catch (InterruptedException e) {
						System.err.println("TaskManager is interrupted.");
						System.err.println("Restart TaskManager.");
					}
				}
				i = taskList.iterator();
				Task t;
				while (i.hasNext()) {
					if ((t = (Task) i.next()).status != Task.QUEUED)
						continue;
					next = t;
					break;
				}
				if (next == null) {
					try {
						this.wait();
					} catch (InterruptedException e) {
						System.err.println("TaskManager is interrupted.");
						System.err.println("Restart TaskManager.");
					}
				}
			}
		}
		return next;
	}

	private void executeTask(Task t) {
		synchronized (this) {
			t.status = Task.INPROGRESS;
			t.startTime = getCurrentTime();
			this.notifyTaskListeners(t);
			System.out.println(Task.TYPENAME[t.type] + " "
					+ Task.STATUSNAME[t.status]);
			t.start();
			while (t.status == Task.INPROGRESS) {
				try {
					this.wait();
				} catch (InterruptedException e) {
					System.err.println("TaskManager is interrupted.");
					System.err.println("Restart TaskManager.");
				}
			}
			t.endTime = getCurrentTime();
			t.duration = getDuration(t.startTime, t.endTime);
			System.out.println(Task.TYPENAME[t.type] + " "
					+ Task.STATUSNAME[t.status]);
			this.notifyTaskListeners(t);
		}
	}

	public void addTaskListener(TaskListener l) {
		this.taskListeners.addElement(l);
	}

	public void removeTaskListener(TaskListener l) {
		this.taskListeners.removeElement(l);
	}

	private void notifyTaskListeners(Task t) {
		Iterator i = taskListeners.iterator();
		while (i.hasNext()) {
			((TaskListener) i.next()).taskUpdated(t, taskList);
		}
	}
}
