package util;

import java.util.Hashtable;

import javax.swing.JTextField;

public class VisDataNode extends VisTreeNode {
	public String datafilter="";
	public String colorfilter="";
	public String originalIndex="";
	public String flowNumber="";
	public boolean isVisible = true;
	public int childDataNodeCounter = 0;
	public int childPlotNodeCounter = 0;
	public Hashtable<String, String> color;
	
	public VisDataNode(String name, boolean isVisible){
		super(name, true);
		this.isVisible = isVisible;
	}
	
	public VisDataNode(String name, String description, boolean isVisible){
		super(name, true, description);
		this.isVisible = isVisible;
	}
	
	public VisDataNode getRootDataNode(){
		return ((VisDataNode)(this.getPath()[1]));
	}
	
	public int getPlotCount(){
		int ret = 0;
		for (int i = 0; i < getChildCount(); i++){
			if (getChildAt(i) instanceof VisPlotNode)
				ret++;
		}
		return ret;
	}
	
	public void setDescription(){
		this.description = "";
		
		this.description += "Data Name: " + this.name + "\n";
		this.description += "Flow Number: " + getHumanReadableNumber(this.flowNumber) + "(" +  this.flowNumber + ")\n";
		this.description += "Data Filter:\n" + getFullDataFilter(this);
		this.description += "Color Filter:\n" + this.colorfilter;
		this.description += "Original Index:\n" + this.originalIndex;
	}
	
	private String getFullDataFilter(VisDataNode parent){
		String filter = parent.getLevel() + ":" + parent.name + ":" + parent.datafilter + "\n";;
		
		if(parent.getLevel() > 1){
			filter = getFullDataFilter((VisDataNode)parent.getParent()) + filter;
		}
		return filter;
	}
	
	public String getDescription(){
		return this.description;
	}
	
	private String getHumanReadableNumber(String in){
		String out = "";
		
		int i=in.length();
		
		if (i >= 10)
			out = in.substring(0, i - 9) + "G";
		else if (i >= 7)
			out = in.substring(0, i - 6) + "M";
		else if (i >= 4)
			out = in.substring(0, i - 3) + "K";
		else
			out = in.toString();
		
		return out;
	}
	
	public void setColor(Hashtable<String, String> color){
		if (color == null) return;
		this.color = (Hashtable<String, String>)color.clone();
		this.colorfilter = getBrushFilter();
	}
	
	private String getBrushFilter() {
		String color = this.color.get("DEFAULT") + "(DEFAULT)\n";
		String tmp;
		
		for (int i = 0; i < 24; i++) {
			if (i == 8)
				i = 16;
			tmp =  this.color.get(String.valueOf(i < 10 ? "0" + i : String.valueOf(i)));
			if (tmp==null)
				continue;
			color += i + "(" + tmp + ") \n";
		}
		return color;
	}
}
